import { write } from "../../output.mjs";
import debug from "debug";
import * as fs from "fs";

/** @type {object} */
const log = debug("css").extend("mixin");

export const command = "mixin";
export const desc =
  "Generate a CSS file that mixes in various prefers-* selectors";

export function builder(argv) {
  argv
    .option("o", {
      alias: "output",
      describe: "The output path for colors.css",
      type: "string",
    })
    .option("s", {
      alias: "selector",
      describe: "The CSS selector to wrap the colors",
      demandOption: true,
      type: "string",
    })
    .option("default", {
      type: "string",
      choices: ["light", "dark"],
      default: "light",
      describe: "The default theme if none are provided",
    })
    .option("light", {
      type: "string",
      describe: "The path to CSS that provides the base light theme",
    })
    .option("light-more-contrast", {
      type: "string",
      describe: "The path to CSS that provides more contrast for light theme",
    })
    .option("light-less-contrast", {
      type: "string",
      describe: "The path to CSS that provides less contrast for light theme",
    })
    .option("dark", {
      type: "string",
      describe: "The path to CSS that provides the base dark theme",
    })
    .option("dark-more-contrast", {
      type: "string",
      describe: "The path to CSS that provides more contrast for dark theme",
    })
    .option("dark-less-contrast", {
      type: "string",
      describe: "The path to CSS that provides less contrast for dark theme",
    });
}

export async function handler(argv) {
  // We like some noise.
  log("generating CSS mixin file");

  // Load in all the files.
  const light = load(argv, "light");
  const lightMoreContrast = load(argv, "light-more-contrast");
  const lightLessContrast = load(argv, "light-less-contrast");
  const dark = load(argv, "dark");
  const darkMoreContrast = load(argv, "dark-more-contrast");
  const darkLessContrast = load(argv, "dark-less-contrast");

  // Figure out the default.
  const auto = argv.default === "light" ? light : dark;

  const autoMoreContrast =
    argv.default === "light" ? lightMoreContrast : darkMoreContrast;

  const autoLessContrast =
    argv.default === "light" ? lightLessContrast : darkLessContrast;

  // Build up the components of the style.
  let lines = [
    block(argv, argv.selector, auto),
    block(argv, '[data-prefers-contrast="more"]', autoMoreContrast),
    block(argv, '[data-prefers-contrast="less"]', autoLessContrast),
    media(
      argv,
      "@media (prefers-color-scheme: light)",
      block(argv, argv.selector, light),
      block(argv, '[data-prefers-contrast="more"]', lightMoreContrast),
      block(argv, '[data-prefers-contrast="less"]', lightLessContrast),
    ),
    media(
      argv,
      "@media (prefers-color-scheme: dark)",
      block(argv, argv.selector, dark),
      block(argv, '[data-prefers-contrast="more"]', darkMoreContrast),
      block(argv, '[data-prefers-contrast="less"]', darkLessContrast),
    ),
    block(argv, '[data-prefers-color-scheme="light"]', light),
    block(
      argv,
      '[data-prefers-color-scheme="light"][data-prefers-contrast="more"]',
      lightMoreContrast,
    ),
    block(
      argv,
      '[data-prefers-color-scheme="light"][data-prefers-contrast="less"]',
      lightLessContrast,
    ),
    block(argv, '[data-prefers-color-scheme="dark"]', dark),
    block(
      argv,
      '[data-prefers-color-scheme="dark"][data-prefers-contrast="more"]',
      darkMoreContrast,
    ),
    block(
      argv,
      '[data-prefers-color-scheme="dark"][data-prefers-contrast="less"]',
      darkLessContrast,
    ),
  ].flat();

  // Write out the results to the console or a file.
  await write(argv, lines);
}

/**
 * Generates a number of CSS blocks.
 *
 * @param argv The arguments.
 * @param {string} selector The CSS selector to put in.
 * @param {string[]} lines The text to put inside the block.
 */
function media(argv, selector, ...lines) {
  // If we have a blank line, then we don't want to write the section.
  lines = lines.flat();

  if (!lines || !lines[0]) {
    return "";
  }

  // We include the selector, which is either a @media query or the user's
  // selector. If we have a @media selector, we want to add the user's selector
  // as the inner block.
  let blockLines = [`${selector} {`];

  // Insert all the lines while indenting them.
  for (const line of lines) {
    blockLines.push(`  ${line}`);
  }

  // Finish up the blocks.
  blockLines.push(`}`);

  return blockLines;
}

/**
 * Generates a number of CSS blocks.
 *
 * @param argv The arguments.
 * @param {string} selector The CSS selector to put in.
 * @param {string[]} lines The text to put inside the block.
 */
function block(argv, selector, ...lines) {
  // If we have a blank line, then we don't want to write the section.
  lines = lines.flat();

  if (!lines || !lines[0]) {
    return "";
  }

  // We include the selector, which is either a @media query or the user's
  // selector. If we have a @media selector, we want to add the user's selector
  // as the inner block.
  let blockLines = [`${selector} {`];

  // Insert the lines with a little indent.
  for (const line of lines) {
    blockLines.push(`  ${line}`);
  }

  // Finish up the blocks.
  blockLines.push(`}`);

  return blockLines;
}

/**
 * Loads a CSS file and removes the pseudo-selector, :theme.
 *
 * @param {object} argv
 * @param {string} key
 * @returns {string | undefined}
 */
function load(argv, key) {
  // See if we have a file to load.
  const value = argv[key];

  if (!value || value.trim() === "") {
    log(`no value provided for --${key}`);

    return undefined;
  }

  // Load the file into memory so we can format it.
  log(`reading --${key}=${argv[key]}`);

  let text = fs.readFileSync(argv[key]).toString().replace(/\r\n/g, "\n");
  const hasThemeSelector = text.match(/^\s*:theme/);

  // If we have a theme selector, we need to get rid of it and the trailing
  // brace but leave any whitespace in front of the line so we can outdent
  // each of the characters.
  if (hasThemeSelector) {
    text = text.replace(/^\s*:theme\s*{\s*\n/, "").replace(/\s*}\s*$/, "");
  }

  // If we only have one line, then trim it and return it.
  const lines = text.split("\n");

  if (lines.length <= 1) {
    return [lines[0].trim()];
  }

  // Otherwise, remove the whitespace in front of each of the lines so the
  // resulting CSS is "prettier".
  const prefix = lines[0].replace(/^(\s*).*$/, "$1");
  const replace = new RegExp(`^${prefix}`);
  const normalized = lines.map((line) => line.replace(replace, ""));

  return normalized;
}
