import * as colors from "../../colors.mjs";
import { write } from "../../output.mjs";
import debug from "debug";

/** @type {object} */
const log = debug("css").extend("variables");

export const command = "variables";
export const desc = "Generate a CSS file that uses variables to generate";

export function builder(argv) {
  argv
    .option("o", {
      alias: "output",
      describe: "The output path for colors.css",
      type: "string",
    })
    .option("s", {
      alias: "selector",
      describe: "The CSS selector to wrap the colors",
      type: "string",
    });
}

export async function handler(argv) {
  // Start with the basic CSS header.
  let lines = [];

  log("generating CSS using variables");

  // Include the selector, if we have one.
  let prefix = "";
  let footer = "";

  if (argv.selector) {
    log("using CSS selector:", argv.selector);
    lines.push(`${argv.selector} {`);
    prefix = "  ";
    footer = "}";
  }

  // We have ten colors, with 0 being the base hue and the others being evenly
  // rotated around the color wheel.
  for (const color of colors.colorList) {
    // Figure out the rotation.
    const rotation = colors.getHueRotation(color);

    // For each hue, we have ten levels of brightness ranging from very dark
    // to very light.
    for (const brightness of colors.brightnessList) {
      // We use a standard code (--color-cXbY) for our codes.
      const code = `--color-priduck-c${color}b${brightness}`;

      // Figure out the ramps we are using for brighteness.
      const l = colors.getLightness(brightness);
      const s = colors.getChroma(brightness);

      // Write out the CSS.
      lines.push(
        `${prefix}${code}: lch(${l.toFixed(1)} ${s.toFixed(
          1,
        )} calc(var(--color-priduck-hue) + ${rotation}));`,
      );
    }
  }

  // Finish up the lines.
  lines.push(footer);

  // Write out the results to the console or a file.
  await write(argv, lines);
}
