import Color from "colorjs.io";

/**
 * Contains the color codes with zero (0) being the base hue.
 *
 * @type {number[]}
 */
export const colorList = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9];

/**
 * Contains the standard brightness codes with zero (0) being the darkest and
 * nine (9) being the lightest.
 *
 * @type {number[]}
 */
export const brightnessList = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9];

/**
 * Gets the degree rotation for the given color code.
 *
 * @param {number} color The color code, in the range of [0-9]
 */
export function getHueRotation(color) {
  return color * 36;
}

/**
 * Gets the hue for a given base hue and color.
 *
 * @param {number} base The base hue, in the range [0-360)
 * @param {number} color The color code, in the range of [0-9]
 */
export function getHue(base, color) {
  return (base + getHueRotation(color)) % 360;
}

/**
 * Gets the lighteness for a given brightness.
 *
 * @param {number} brightness The brightenss, in the range of [0-9]
 */
export function getLightness(brightness) {
  return 5 + brightness * 10;
}

/**
 * Gets the saturation for a given brightness.
 *
 * @param {number} brightness The brightenss, in the range of [0-9]
 */
export function getChroma(brightness) {
  return 50 - brightness * 5;
}

/**
 * Get the LCH string
 *
 * @param {number} base The base hue, in the range [0-360)
 * @param {number} color The color code, in the range of [0-9]
 * @param {number} brightness The brightenss, in the range of [0-9]
 */
export function getLchCss(base, color, brightness) {
  const l = getLightness(brightness);
  const c = getChroma(brightness);
  const h = getHue(base, color);

  return `lch(${l}% ${c} ${h})`;
}

/**
 * Gets the RGB values as a string hex of a hash plus 6 numbers.
 *
 * @param {number} base The base hue, in the range [0-360)
 * @param {number} color The color code, in the range of [0-9]
 * @param {number} brightness The brightenss, in the range of [0-9]
 */
export function getRgbHex(base, color, brightness) {
  const lch = getLchCss(base, color, brightness);
  const lchColor = new Color(lch);
  const hex = lchColor
    .to("srgb")
    .toString({ format: "hex", inGamut: true })
    .replace(/^#(\d)(\d)(\d)$/, "#$1$1$2$2$3$3");

  return hex;
}

/**
 * Gets the RGB values as an array of three numbers.
 *
 * @param {number} base The base hue, in the range [0-360)
 * @param {number} color The color code, in the range of [0-9]
 * @param {number} brightness The brightenss, in the range of [0-9]
 */
export function getRgbArray(base, color, brightness) {
  const hex = getRgbHex(base, color, brightness);
  const r = parseInt(hex.substring(1, 3), "16");
  const g = parseInt(hex.substring(3, 5), "16");
  const b = parseInt(hex.substring(5, 7), "16");

  return [r, g, b];
}
